<?php
// +------------------------------------------------------------------------------+
// | MBAPI [TM] .:. Client Billing Engine                                         |
// +------------------------------------------------------------------------------+
// | Copyright (c) 2001-2006 ModernGigabyte, LLC                                  |
// +------------------------------------------------------------------------------+
// | This source file is subject to the ModernGigabyte End User License Agreement |
// | (EULA), that is bundled with this package in the file LICENSE, and is        |
// | available at through the world-wide-web at http://www.mbapi.com/eula.php     |
// |                                                                              |
// | If you did not receive a copy of the MBAPI license and are unable to obtain  |
// | it through the world-wide-web, please send a note to license@mbapi.com so we |
// | can email you a copy immediately.                                            |
// +------------------------------------------------------------------------------+
// | Authors: ModernGigabyte, LLC <info@moderngigabyte.com>                       |
// | Support: http://www.modernsupport.com/                                       |
// +------------------------------------------------------------------------------+
// | ModernGigabyte, ModernBill, and MBAPI are trademarks of ModernGigabyte, LLC. |
// +------------------------------------------------------------------------------+

require_once dirname(__FILE__)."/../../../../lib-tk/shield.php";
require_once mb_path("/lib-action/action.php");
require_once mb_path("/lib-mbapi/mbapi.php");

/**
 * This action displays the form to enter an activation key for KA
 */
class RetrieveKeysResponse extends MB_Action {

	function RetrieveKeysResponse() {
		parent::MB_Action();
		$this->stringRep = "RetrieveKeysResponse";
	}

	function execute() {
		global $CONF;
		
		$keyHistory = array();
		$keys = array_merge(arrayFor(":configParams", array(
			"configGroupID" => internalConfigGroupID,
			"configParamName" => array("license"),
			"sortDir" => "DESC",
			"sortColumn" => "configParamID",
		)), arrayFor(":configParams", array(
			"configGroupID" => internalConfigGroupID,
			"configParamName" => array("license_archived"),
			"sortDir" => "DESC",
			"sortColumn" => "configParamID",
		)));
		
		$current = new OPEN_FUSION_LICENSE(CURRENT_LICENSE_KEY);
		if ($current->licenseParsed){
			$keyNumber = $current->keyNumber;
			$keyDomain = $current->licenseDomain;
			$licenseServerURL = $current->licenseServerURL;
		}
		
		foreach ((array)$keys as $k){
			$x = new OPEN_FUSION_LICENSE($k['configParamValue']);
			if ($x->licenseParsed){ // not interested in licenses that don't parse :-)
				$keyHistory[] = array(
					"keyNumber" => $x->keyNumber.".".$x->keyVersion,
					"updatePassword" => $x->updateTicket,
				);
			}
		}
		
		if (!$keyHistory){
			$this->smartyVars['success'] = false;
			$this->smartyVars['errorText'] = "You must first activate a license or upload a license before you can update it.";
		} else {
		
			dbg("INITIATING KA PKP REQUEST");
			$response = KA_PKP_Request::getKey(
				KAPRODUCTU,
				KAPRODUCTP,
				$keyHistory,
				createPPBInfoStruct(),
				array(
					'targetProductInfo' => (object)array(
						'productName' => 'plesk-billing',
						'productVersion' => AppParentVersion,
					),
					'guid' => $keyDomain,
				),
				$licenseServerURL
			);
			
			if ($response->cleanKeysHistory){
				dbg("was told to clean keys history, removing all old keys...");
				foreach ((array)$keys as $k){
					if ($k["configParamID"]!=CURRENT_KEY_CONFIG_PARAM_ID){
						dbg("removing key: ".$k["configParamID"]);
						deleteForKey(":configParams", $k["configParamID"]);
					}
				}
			}
	
			$key = $response->getKey();
			if ($response->detailResultCode==11){
				$this->smartyVars['upToDate'] = 1;
				$this->smartyVars['success'] = true;
			} elseif ($key->licenseParsed){
				if ($response->cleanKeysHistory){
					deleteForKey(":configParams", CURRENT_KEY_CONFIG_PARAM_ID);
				}
				$key->store();
				$this->smartyVars['success'] = true;
			} else {
				$this->smartyVars['success'] = false;
				switch ($response->resultCode){
					case 0:
					case 1:
						$this->smartyVars['errorText'] = $response->resultDesc;
						break;
					case 2:
						$this->smartyVars['errorText'] = "There has been an internal error.  Please try again later.  If you continue to have problems please contact support and give them error code <strong>".$response->detailResultCode."</strong>.";
						break;
					default:
						$this->smartyVars['errorText'] = "There has been an internal error.  Please contact support.";
						break;
				}
			}
		}
		
		$this->addTemplate("modules/RetrieveKeysResponse");

		$this->addErrorTemplates();
		return $this->hasExecutedSuccessfully();

	}
} // RetrieveKeys
