<?php

require_once mb_path("/lib-billing/include/config/internal/statuses.php");
require_once mb_path("/lib-tk/include/config/internal/statuses.php");
require_once mb_path("/lib-tk/include/crypt/crypt.php");

/**
 * modernbill-client specific implementation of operations required by ProcessSSOAPIRequest 
 */
class ClientSSOAPI {
    
    /**
     * Chceck client contact's credentials
     *
     * @param string $name User name
     * @param string $password User's password. If this parameter is empty then only check that user name is correct. 
     * @return String User name identifier (login name).
     */
    function checkCredentials($name, $password) {
        $h = new QueryMaster('select', "clients.client_id");
        $h->addJoin("clients", "client_id", "client_contacts");
        $h->addAndClause("client_contacts.contact_username", "=", $name);
        $h->addAndClause("client_contacts.contact_active", "=", 1, "i");
        $h->addAndClause("clients.client_active", "=", 1, "i");
        $h->addAndClause("clients.client_status", "=", getAPIStatusID("client", "active"), "i");

        if ($password) {
            $h->addAndClause("client_contacts.contact_password", "=", getPasswordHash($password));
        } else {
            // Only check that identifier is correct
        }
        $data = $h->getRow();
        if (count($data)) {
            return $name;
        } else {
            return null;
        }    
    }

    /**
     * Close session by client contact's user name.
     *
     * @param string $userName
     */
    function closeSessionByName($userName) {
        $qh = new QueryMaster('select');
        $qh->addTables('client_contacts');
        $qh->addColumns('client_contact_id');
        $qh->addOrClause('contact_username', '=', $userName);
        $qh->addOrClause('contact_email', '=', $userName);
        foreach ($qh->getResults() as $row) {
            $userId = $row['client_contact_id'];

            $h = new QueryMaster('update');
            $h->addTables('sessions');
            $h->addParam('session_active', 0, 'i');
            $h->addAndClause('user_id', '=', $userId, 'i');
            $h->addAndClause('user_type', '=', 1, 'i');
            $h->execute();            
        }        
    }
} // ClientSSOAPI

?>