<?php

/**
 * Include path lookup does not work well in all setups. So we will do it explicitly.
 * @param string $className
 * @return string
 */
function autoload_lookup_file($className) {
    $fileNameBody = str_replace('_', DIRECTORY_SEPARATOR, $className);
    foreach (array_filter(explode(PATH_SEPARATOR, get_include_path())) as $base) {
        $fileName = $base . DIRECTORY_SEPARATOR . $fileNameBody . '.php';        
        if (is_file($fileName)) {
            return $fileName;    
        }
    }
    return null;
}


/**
 * This function allows to get class description without dying in case class does not exist.
 * TODO (refactoring) Probably this function sould be somewhere in utilities.
 * @param string $className 
 * @return ReflectionClass Reflection class or null depending whether class definition can be found.
 */
function autoload_describe_class($className) {
    if (!autoload_lookup_file($className)) {
        return null;
    }
    try {        
        return new ReflectionClass($className);
    } catch (ReflectionException $e) {
        lib_Config::getLog()->info("Can not instantiate class $className", $e );
        return null;
    }
}


/**
 * Exceptions that are thrown from __autoload function can not be caught. 
 * So this function makes best effort to write message or terminates everyting immediately.
 *
 * @param string $message
 */
function autoload_log_message($message) {
    // (Note that log itself depends on autoload).    
    @include_once('lib/Log.php');
    if (@include_once('lib/Log.php')) {
        lib_Config::getLog()->error("Class autoload failure at " . var_export(array_slice(debug_backtrace(), 0, 3), true) . ")");
        // echo "Class autoload failure at " . var_export(array_slice(debug_backtrace(), 0, 3), true) . ")"; // DEBUG
        lib_Config::getLog()->fatal_error($message);
    } else {
        // Fallback code that does not use logger fcility.
        echo($message);
        die(1);
    }
}

/**
 * Class autoloader.
 * Re-define or extend include path as necessary.
 * 
 * Note that expceptions ar not thrown from this function as they are not handled correctly. 
 * @param string $className
 */
function __autoload($className) {
    try {
        assert('$className'); // is not empty
        $fileName = autoload_lookup_file($className);
        if (is_null($fileName)) {
            autoload_log_message(
                "Can not find appropriate file for class $className. Include path is "
                . get_include_path() );
        }
        /*
         * 'require_once' is inappropriate here as it may (indirectly via error handler)
         * throw exception which will not be handled.
         */
        // XXX If file loaded by following include operator contain errors then PHP dies silently.
        include_once($fileName); // always returns TRUE if successfully called more than one time
        if (!@include_once($fileName)) {
            autoload_log_message("Can not load '$fileName' for class '$className'");                
        } else {
            ; // lib_Config::getLog()->trace("Loaded file $fileName for class $className.");
        }
    } catch (Exception $e) {
        autoload_log_message(
            "Can not autoload load '$fileName':"
            . " exception at {$e->getFile()}({$e->getLine()}) message '{$e->getMessage()}' trace " . $e->getTraceAsString());
    }
}

?>