<?php
/*
 * This file contains miscellaneous utility functions.
 *
 * TODO (refactoring) Put these functions in a class so autoloading can be used.
 */

function prn() {
    foreach (func_get_args() as $arg) {
        echo $arg . " ";
    }
    echo "\n";
}

function in_range($value, $min, $max) {
    return $min <= $value && $value < $max;
}

/**
 * @return string String representation of current time.
 */
function timeAsString() {
    return gmdate('Y-m-d\TH:i:s\Z');
}


function make_boolean_string($value) {
    if ($value) {
        return 'true';
    } else {
        return 'false';
    }
}


function begins_with($str, $sub) {
    return strncmp($str, $sub, strlen($sub)) === 0;
}


/**
 * This function avoids those shitty warnings about absent keys in cases when they
 * may be indeed absent and that's normal. Hey, PHP, breath deeper, keep calm,
 * try to relax. I know what I am doing, trust me.
 *
 * @param array $assoc_array
 * @param mixed $index
 * @return mixed
 */
function get_at($assoc_array, $index, $default = null) {
    assert('is_array($assoc_array)');
    if (array_key_exists($index, $assoc_array)) {
        return $assoc_array[$index];
    } else {
        return $default;
    }
}


function unpack_value($packed_value) {
    return unserialize(gzuncompress(base64_decode($packed_value)));
}


function pack_value($value) {
    return base64_encode(gzcompress(serialize($value)));
}


function rmdir_r($dir) {
    if (!is_readable($dir)) {
        return false;
    }
    $handle = opendir($dir);
    while (false !== ($item = readdir($handle))) {
        if ($item != '.' && $item != '..') {
            if (is_dir($dir . DIRECTORY_SEPARATOR . $item)) {
                rmdir_r($dir . DIRECTORY_SEPARATOR . $item);
            } else {
                unlink($dir . DIRECTORY_SEPARATOR . $item);
            }
        }
    }
    closedir($handle);
    if (rmdir($dir)) {
        $success = true;
    }
    return $success;
}


/**
 * Uniqid is supplemented by pseudorandoms as it uses nanotime as source but
 * in most cases we need identifiers that are hard to guess.
 *
 * @param int $min_length
 * @return string
 */
function make_new_id($min_length = 32) {
    $result = base_convert(uniqid(), 16, 36);
    // uniqid is relatively deterministic part so it is made suffix to help hashing
    while (strlen($result) < $min_length) {
        $result = base_convert(mt_rand(), 10, 36) . $result;
    }
    // A letter is prepended so identifier is a valid xml:id and programming language id.
    return chr(mt_rand(ord('a'), ord('z'))) . $result;
}


function make_new_uuid() {
    $hash = make_new_id();
    return substr($hash, 0, 8) . '-' . substr($hash, 8, 4) . '-'
    . substr($hash, 12, 4). '-' . substr($hash, 16, 4). '-' . substr($hash, 20);
}


/**
 * Strip given prefix from a string. If string has no such prefix then return null.
 *
 * @param string $a_string
 * @param string $prefix
 * @return string
 */
function strip_prefix($a_string, $prefix) {
    if (0 === strlen($prefix)) {
        return $a_string;
    } else if (0 === strpos($a_string, $prefix, 0)) {
        if (strlen($prefix) === strlen($a_string)) {
            return "";
        } else {
            return substr($a_string, strlen($prefix));
        }
    } else {
        return null;
    }
}


function escape_eols($str) {
    return preg_replace('|[\n\r]+|', '\n', $str);
}


function ellipsis_truncate($str, $max_len) {
    $elp = '...';
    if (strlen($str) > $max_len && strlen($elp) < $max_len) {
        return substr($str, 0, $max_len - strlen($elp)) . $elp;
    } else {
        return $str;
    }
}


/**
 * Remove signature and keys delimiters like '-----BEGIN CERTIFICATE-----'
 *
 * @param string $text
 * @param string $block_name e.g. 'CERTIFICATE'
 * @return string
 */
function trim_block_delimiters($text, $block_name) {
    $dashes = '-----';
    $result = ereg_replace($dashes.'BEGIN '.$block_name.$dashes, '', $text);
    $result = ereg_replace($dashes.'END '.$block_name.$dashes, '', $result);
    // echo $dashes.'BEGIN '.$block_name.' '.$dashes; // DEBUG
    return trim(ereg_replace("\n", '', $result));
    // return trim($result);
}


function list_to_string($list) {
    // I like the way it looks in Python and Haskell:
    return '[' . implode(', ', $list) . ']';
}


function array_update(&$lst, $new_lst) {
    $lst = array_merge($lst, $new_lst);
}


function is_base64_encoded($data) {
    return 0 != preg_match('|^[a-zA-Z0-9/+]*={0,2}$|', $data);
}


function get_obj_props($object, $propNames) {
    $result = array();
    foreach ($propNames as $propName) {
        $result[$propName] = $object->$propName;
    }
    return $result;
}

function array_sort($array) {
    sort($array);
    return $array;

}

?>